<?php
/**
 * @connect_module_class_name CPayanyway
*/

if (!class_exists('CPayanyway')) {
	
	class CPayanyway extends PaymentModule
	{

		var $type = PAYMTD_TYPE_ONLINE;
		var $additionalParameters = '';
		var $isInvoice = 0;
		var $paymentModuleID = '';

		function _initVars()
		{
			parent::_initVars();
			$this->title 		= PAW_PAYANYWAY_TTL;
			$this->description 	= PAW_PAYANYWAY_DSCR;
			$this->sort_order 	= 9;
			$this->Settings = array(
				'CONF_PAYANYWAY_SYSTEMURL',
				'CONF_PAYANYWAY_MNTID',
				'CONF_PAYANYWAY_DATAINTEGRITYCODE',
				'CONF_PAYANYWAY_TRANS_CURRENCY',
				'CONF_PAYANYWAY_TESTMODE',
				'CONF_PAYANYWAY_ORDERSTATUS',
				'CONF_PAYANYWAY_LOGIN',
				'CONF_PAYANYWAY_PASSWORD'
			);

			$ModuleConfigs = current(modGetModuleConfigs(get_class()));
			$this->paymentModuleID = isset($ModuleConfigs['ConfigID'])?$ModuleConfigs['ConfigID']:0;
		}

		function _initSettingFields()
		{
			$this->SettingsFields['CONF_PAYANYWAY_SYSTEMURL'] = array(
				'settings_value' 		=> 'www.payanyway.ru',
				'settings_title' 		=> PAW_CFG_MNT_SERVER_TTL,
				'settings_description' 	=> PAW_CFG_MNT_SERVER_DCSR,
				'settings_html_function'=> 'setting_TEXT_BOX(0,',
				'sort_order' 			=> 1,
			);
			$this->SettingsFields['CONF_PAYANYWAY_MNTID'] = array(
				'settings_value' 		=> '',
				'settings_title' 		=> PAW_CFG_MNT_ID_TTL,
				'settings_description' 	=> PAW_CFG_MNT_ID_DSCR,
				'settings_html_function'=> 'setting_TEXT_BOX(0,',
				'sort_order' 			=> 1,
			);
			$this->SettingsFields['CONF_PAYANYWAY_DATAINTEGRITYCODE'] = array(
				'settings_value' 		=> '',
				'settings_title' 		=> PAW_CFG_MNT_DATAINTEGRITY_CODE_TTL,
				'settings_description' 	=> PAW_CFG_MNT_DATAINTEGRITY_CODE_DSCR,
				'settings_html_function'=> 'setting_TEXT_BOX(0,',
				'sort_order' 			=> 1,
			);
			$this->SettingsFields['CONF_PAYANYWAY_TRANS_CURRENCY'] = array(
				'settings_value' 		=> '',
				'settings_title' 		=> PAW_CFG_MNT_CURRENCY_TTL,
				'settings_description' 	=> PAW_CFG_MNT_CURRENCY_DSCR,
				'settings_html_function'=> 'setting_CURRENCY_SELECT(',
			);
			$this->SettingsFields['CONF_PAYANYWAY_TESTMODE'] = array(
				'settings_value' 		=> '0',
				'settings_title' 		=> PAW_CFG_MNT_TEST_MODE_TTL,
				'settings_description' 	=> PAW_CFG_MNT_TEST_MODE_DSCR,
				'settings_html_function'=> 'setting_CHECK_BOX(',
				'sort_order' 			=> 1,
			);
			$this->SettingsFields['CONF_PAYANYWAY_ORDERSTATUS'] = array(
				'settings_value' 		=> '',
				'settings_title' 		=> PAW_CFG_MNT_ORDER_STATUS_TTL,
				'settings_description' 	=> PAW_CFG_MNT_ORDER_STATUS_DSCR,
				'settings_html_function'=> 'setting_ORDER_STATUS_SELECT(',
				'sort_order' 			=> 1,
			);
			$this->SettingsFields['CONF_PAYANYWAY_LOGIN'] = array(
				'settings_value' 		=> '',
				'settings_title' 		=> PAW_CFG_LOGIN_TTL,
				'settings_description' 	=> PAW_CFG_LOGIN_DSCR,
				'settings_html_function'=> 'setting_TEXT_BOX(0,',
				'sort_order' 			=> 1,
			);
			$this->SettingsFields['CONF_PAYANYWAY_PASSWORD'] = array(
				'settings_value' 		=> '',
				'settings_title' 		=> PAW_CFG_PASSWORD_TTL,
				'settings_description' 	=> PAW_CFG_PASSWORD_DSCR,
				'settings_html_function'=> 'setting_TEXT_BOX(0,',
				'sort_order' 			=> 1,
			);
		}

		function after_processing_html( $orderID ) 
		{
			if (!isset($_REQUEST['invoice']))
			{
				if ($this->isInvoice) {
					$action = "index.php?order4_confirmation_quick=yes&"."order_success=yes&paymentMethodID=".$_GET["paymentMethodID"]."&orderID=".$orderID."&invoice=yes";
				} else {
					$action = "https://{$this->_getSettingValue('CONF_PAYANYWAY_SYSTEMURL')}/assistant.htm";
				}

				$mnt_id = (int)$this->_getSettingValue('CONF_PAYANYWAY_MNTID');
				$mnt_test_mode = (int)$this->_getSettingValue('CONF_PAYANYWAY_TESTMODE');
				$mnt_dataintegrity_code = $this->_getSettingValue('CONF_PAYANYWAY_DATAINTEGRITYCODE');
				$currency = currGetCurrencyByID($this->_getSettingValue('CONF_PAYANYWAY_TRANS_CURRENCY'));
				$mnt_currency_code = ($currency['currency_iso_3'] == "RUR") ? "RUB" : $currency['currency_iso_3'];
				$order = ordGetOrder( $orderID );
				$order_amount = _formatPrice(PaymentModule::_convertCurrency($order['order_amount'],0,$this->_getSettingValue('CONF_PAYANYWAY_TRANS_CURRENCY')), 2, '.', '');
				$mnt_signature	 = md5("{$mnt_id}{$orderID}{$order_amount}{$mnt_currency_code}{$mnt_test_mode}{$mnt_dataintegrity_code}");

				$payment_method = get_class(modGetModuleObj($this->paymentModuleID));
				$ModuleConfigs = current(modGetModuleConfigs($payment_method));
				$currentPaymentModule = modGetModuleObj( $ModuleConfigs['ConfigID'], PAYMENT_MODULE );

				$unitid = $currentPaymentModule->_getSettingValue('CONF_PAYANYWAY_UNIT_ID');
				if ($unitid)
				{
					$this->additionalParameters .= "<input type=hidden name=\"paymentSystem.unitId\" value=\"{$unitid}\">\n";
					$this->additionalParameters .= "<input type=hidden name=\"paymentSystem.limitIds\" value=\"{$unitid}\">\n";
				}
				$accountid = $currentPaymentModule->_getSettingValue('CONF_PAYANYWAY_ACCOUNT_ID');
				if ($accountid)
					$this->additionalParameters .= "<input type=hidden name=\"paymentSystem.accountId\" value=\"{$accountid}\">\n";

				$res = "";
				$res .= 
					"<table width='100%'>\n".
					"	<tr>\n".
					"		<td align='center'>\n".
					"		<form method=\"POST\" action=\"{$action}\">\n".
					"		<input type=hidden name=\"MNT_ID\" value=\"{$mnt_id}\">\n".
					"		<input type=hidden name=\"MNT_TRANSACTION_ID\" value=\"{$orderID}\">\n".
					"		<input type=hidden name=\"MNT_CURRENCY_CODE\" value=\"{$mnt_currency_code}\">\n".
					"		<input type=hidden name=\"MNT_AMOUNT\" value=\"{$order_amount}\">\n".
					"		<input type=hidden name=\"MNT_TEST_MODE\" value=\"{$mnt_test_mode}\">\n".
					"		<input type=hidden name=\"MNT_SIGNATURE\" value=\"{$mnt_signature}\">\n".
							$this->additionalParameters.
					"		<input type=hidden name=\"MNT_SUCCESS_URL\" value=\"".getTransactionResultURL('success')."\">\n".
					"		<input type=hidden name=\"MNT_FAIL_URL\" value=\"".getTransactionResultURL('failure')."\">\n".
					"		<input type=hidden name=\"followup\" value=\"true\">\n".
					"		<input type=hidden name=\"javascriptEnabled\" value=\"true\">\n".
					"		<input type=submit value=\"    PayAnyWay !\"><br>\n".
					"		</form>\n".
					"		</td>\n".
					"	</tr>\n".
					"</table>";
			} 
			else 
			{
				$payment_method = get_class(modGetModuleObj($this->paymentModuleID));

				$ModuleConfigs = current(modGetModuleConfigs('CPayanyway'));
				$currentPaymentModule = modGetModuleObj( $ModuleConfigs['ConfigID'], PAYMENT_MODULE );

				$login = iconv('ISO-8859-1', 'UTF-8', $currentPaymentModule->_getSettingValue('CONF_PAYANYWAY_LOGIN'));
				$password = iconv('ISO-8859-1', 'UTF-8', $currentPaymentModule->_getSettingValue('CONF_PAYANYWAY_PASSWORD'));

				require_once(dirname(__FILE__).'/MonetaAPI/MonetaWebService.php');

				switch ($currentPaymentModule->_getSettingValue('CONF_PAYANYWAY_SYSTEMURL'))
				{
					case "demo.moneta.ru":
						$service = new MonetaWebService("https://demo.moneta.ru/services.wsdl", $login, $password);
						break;
					case "www.payanyway.ru":
						$service = new MonetaWebService("https://service.payanyway.ru/services.wsdl", $login, $password);
						break;
				}

				$smarty = new Smarty();
				$smarty->template_dir = "core/modules/tpl/";

				try
				{
					$totalAmount = $_REQUEST['MNT_AMOUNT']." ".$_REQUEST['MNT_CURRENCY_CODE'];
					$fee = "-";

					//    
					if ($_REQUEST['paymentSystem_accountId']) {
						$transactionRequestType = new MonetaForecastTransactionRequest();
						$transactionRequestType->payer = $_REQUEST['paymentSystem_accountId'];
						$transactionRequestType->payee = $_REQUEST['MNT_ID'];
						$transactionRequestType->amount = $_REQUEST['MNT_AMOUNT'];
						$transactionRequestType->clientTransaction = $_REQUEST['MNT_TRANSACTION_ID'];
						$forecast = $service->ForecastTransaction($transactionRequestType);

						$totalAmount = number_format($forecast->payerAmount,2,'.','')." ".$forecast->payerCurrency;
						$fee = number_format($forecast->payerFee,2,'.','')." ".$forecast->payerCurrency;
					}

					$request = new MonetaInvoiceRequest();
					if ($_REQUEST['paymentSystem_accountId'])
						$request->payer = $_REQUEST['paymentSystem_accountId'];
					$request->payee = $_REQUEST['MNT_ID'];
					$request->amount = $_REQUEST['MNT_AMOUNT'];
					$request->clientTransaction = $_REQUEST['MNT_TRANSACTION_ID'];

					if ($payment_method == 'CPayanywayPost')
					{
						$operationInfo = new MonetaOperationInfo();
						$a = new MonetaKeyValueAttribute();
						$a->key = 'mailofrussiaindex';
						$a->value = iconv('ISO-8859-1', 'UTF-8', $_REQUEST['additionalParameters_mailofrussiaSenderIndex']);
						$operationInfo->addAttribute($a);
						$a1 = new MonetaKeyValueAttribute();
						$a1->key = 'mailofrussiaregion';
						$a1->value = iconv('ISO-8859-1', 'UTF-8', $_REQUEST['additionalParameters_mailofrussiaSenderRegion']);
						$operationInfo->addAttribute($a1);
						$a2 = new MonetaKeyValueAttribute();
						$a2->key = 'mailofrussiaaddress';
						$a2->value = iconv('ISO-8859-1', 'UTF-8', $_REQUEST['additionalParameters_mailofrussiaSenderAddress']);
						$operationInfo->addAttribute($a2);
						$a3 = new MonetaKeyValueAttribute();
						$a3->key = 'mailofrussianame';
						$a3->value = iconv('ISO-8859-1', 'UTF-8', $_REQUEST['additionalParameters_mailofrussiaSenderName']);
						$operationInfo->addAttribute($a3);
						$request->operationInfo = $operationInfo;
					}
					elseif ($payment_method == 'CPayanywayEuroset')
					{
						$operationInfo = new MonetaOperationInfo();
						$a1 = new MonetaKeyValueAttribute();
						$a1->key = 'rapidamphone';
						$a1->value = $_REQUEST['additionalParameters_rapidaPhone'];
						$operationInfo->addAttribute($a1);
						$request->operationInfo = $operationInfo;
					}
					$response = $service->Invoice($request);
					if ($payment_method == 'CPayanywayEuroset')
					{
						$response1 = $service->GetOperationDetailsById($response->transaction);
						foreach ($response1->operation->attribute as $attr)
						{
							if ($attr->key == 'rapidatid')
							{
								$transaction_id = $attr->value;
							}
						}
					}
					else
					{
						$transaction_id = $response->transaction;
					}
					
					$smarty->assign('title', PAW_INVOICE_CREATED_TTL);
					$smarty->assign('invoice', array(
						'status' => $response->status,
						'system' => $payment_method,
						'unitid' => $_REQUEST['paymentSystem_unitId'],
						'paymentserver' => $currentPaymentModule->_getSettingValue('CONF_PAYANYWAY_SYSTEMURL'),
					    'transaction' => str_pad($transaction_id, 10, "0", STR_PAD_LEFT),
						'operation' => $response->transaction,
						'amount' => $_REQUEST['MNT_AMOUNT']." ".$_REQUEST['MNT_CURRENCY_CODE'],
						'payerAmount' => $totalAmount,
						'payerFee' => $fee
					));

				}
				catch (Exception $e)
				{
					$smarty->assign('title', PAW_INVOICE_ERROR_TTL);
					$smarty->assign('invoice', array( 'status' => 'FAILED',
													  'error_message' => iconv('utf-8','Windows-1251',$e->getMessage())));
				}			
				$res = $smarty->fetch("payanyway_invoice.tpl.html");
			}

			return $res;
		}

		//  :
		const paw_kassa_VAT0     = 1104;  //  0%
		const paw_kassa_VAT10    = 1103;  //  10%
		const paw_kassa_VAT18    = 1102;  //  18%
		const paw_kassa_VATNOVAT = 1105;  //   
		const paw_kassa_VATWR10  = 1107;  //   .  10%
		const paw_kassa_VATWR18  = 1106;  //   .  18%

		private static function jsonRemoveUnicodeSequences($struct) {
			return preg_replace("/\\\\u([a-f0-9]{4})/e", "iconv('UCS-4LE','UTF-8',pack('V', hexdec('U$1')))", json_encode($struct));
		}

		// paw_kassa_sale -  ""    kassa.payanyway.ru,      PayURL   
		//                   (SUCCESS)
		//
		// :
		// $mnt_id -   
		// $mnt_transaction_id - id 
		// $mnt_amount -   (  Moneta.ru, : 100.00)
		// $mnt_data_integrity_code -    
		// $client_email -    
		// $inventoryPositions = array( array('name' => 'tovar 1', 'price' => 1, 'quantity' => 2, 'vatTag' => paw_kassa_VATNOVAT),
		//                              array('name' => 'tovar 2', 'price' => 3, 'quantity' => 1, 'vatTag' => paw_kassa_VATNOVAT) ) -  
		//      name -  
		//         price -    
		//         quantity -   
		//         vatTag -  
		//
		private static function paw_kassa_sale($mnt_id, $mnt_transaction_id, $mnt_amount, $mnt_data_integrity_code, $client_email, $inventoryPositions)
		{
			$productsTototal = 0;
			foreach ($inventoryPositions AS $item) {
		            $productsTototal = $productsTototal + floatval($item['price']) * floatval($item['quantity']);
		        }

			if (floatval($productsTototal) != floatval($mnt_amount)) {
			    $discountRate = floatval($mnt_amount) / floatval($productsTototal);
		            $newInvenrory = array();
		            foreach ($inventoryPositions AS $item) {
		                $item['price'] = round(floatval($item['price']) * $discountRate, 5);
		                $newInvenrory[] = $item;
		            }
		            $inventoryPositions = $newInvenrory;
			}


			$data = array(
				'id' => $mnt_transaction_id,
				'checkoutDateTime' => date(DATE_ATOM),
				'docNum' => $mnt_transaction_id,
				'docType' => 'SALE',
				'email' => $client_email,
				'responseURL' => 'https://kassa.payanyway.ru/index.php?do=callback',
				'inventPositions' => $inventoryPositions,
				'moneyPositions' => array(array('paymentType' => 'CARD', 'sum' => $mnt_amount))
			);
			$data['signature'] = md5($data['id'].$data['checkoutDateTime'].$mnt_data_integrity_code);
			$jsonData = self::jsonRemoveUnicodeSequences($data);
			$operationUrl = 'https://kassa.payanyway.ru/api/api.php?method=sale&accountid='.$mnt_id;
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $operationUrl);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $jsonData);
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
				'Content-Type: application/json',
				'Content-Length: '.strlen($jsonData)
			));
			$result = curl_exec($ch);
			curl_close($ch);
			return $result;
		}

		function after_payment_php( $orderID, $_REQUEST)
		{
			if (isset($_REQUEST['MNT_ID']) && isset($_REQUEST['MNT_TRANSACTION_ID']) && isset($_REQUEST['MNT_OPERATION_ID'])
				&& isset($_REQUEST['MNT_AMOUNT']) && isset($_REQUEST['MNT_CURRENCY_CODE']) && isset($_REQUEST['MNT_TEST_MODE'])
				&& isset($_REQUEST['MNT_SIGNATURE']))
			{
				$orderID = intval($_REQUEST['MNT_TRANSACTION_ID']);
				if ($orderID && ($order = _getOrderById($orderID))) {
					$mnt_dataintegrity_code = $this->_getSettingValue('CONF_PAYANYWAY_DATAINTEGRITYCODE');
					$signature = md5("{$_REQUEST['MNT_ID']}{$_REQUEST['MNT_TRANSACTION_ID']}{$_REQUEST['MNT_OPERATION_ID']}{$_REQUEST['MNT_AMOUNT']}{$_REQUEST['MNT_CURRENCY_CODE']}{$_REQUEST['MNT_TEST_MODE']}{$mnt_dataintegrity_code}");
					if ($_REQUEST['MNT_SIGNATURE'] == $signature){
						$statusID = (int)$this->_getSettingValue('CONF_PAYANYWAY_ORDERSTATUS');
						$comment = sprintf('   PayAnyWay.   - %d',$orderID);
						ostSetOrderStatusToOrder( $_REQUEST['MNT_TRANSACTION_ID'], $statusID,$comment,0,true);
						// kassa.payanyway.ru support:
						$client_email = iconv('cp1251','utf-8',$order['customer_email']);
						$inventoryPositions = array();
						$discount = $order[9];
						$shipAmount = number_format(round($order[8]*$order[12],2),2,'.','');
						$inventoryPositions[] = array(
							'name' => iconv('cp1251','utf-8',': '.$order[4]),
							'price' => $shipAmount,
							'quantity' => '1',
							'vatTag' => self::paw_kassa_VATNOVAT
						);
						$amount = floatval($shipAmount);
						$orderContent = ordGetOrderContent($orderID);
						foreach ($orderContent as $product)
						{
							$price = number_format(round(floor(round($product['Price']*$order[12], 2)*(1-$discount*0.01)*100)*0.01, 2), 2, '.', '');
							$inventoryPositions[] = array(
								'name' => iconv('cp1251','utf-8',$product['name']),
								'price' => $price,
								'quantity' => $product['Quantity'],
								'vatTag' => self::paw_kassa_VATNOVAT
							);
							$amount += floatval($price * $product['Quantity']);
						}
						$amount = round($amount, 2);
						if ($amount < floatval($_REQUEST['MNT_AMOUNT']))
						{
							$inventoryPositions[] = array(
								'name' => iconv('cp1251','utf-8',''),
								'price' => number_format(round(floatval($_REQUEST['MNT_AMOUNT']) - $amount, 2), 2, '.', ''),
								'quantity' => '1',
								'vatTag' => self::paw_kassa_VATNOVAT
							);
						}
						$result = self::paw_kassa_sale($_REQUEST['MNT_ID'],$_REQUEST['MNT_TRANSACTION_ID'],$_REQUEST['MNT_AMOUNT'],$mnt_dataintegrity_code,$client_email,$inventoryPositions);
						die("SUCCESS");
					} else {
						die("FAIL");
					}
				} else {
					die("FAIL");
				}
			} else {
				die("FAIL");
			}
		}

	}

}
?>
